#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Use of this source code is governed by a BSD-style
# license that can be found in the LICENSE file.
# Copyright 2019 The logging-gelf Authors. All rights reserved.

import socket
import ssl
from logging.handlers import SocketHandler, DatagramHandler
import platform


class GELFTCPSocketHandler(SocketHandler):
    """GELFTCPSocketHandler"""

    def __init__(self, host, port, use_tls=False, cert_reqs=ssl.CERT_NONE,
                 ca_certs=None):
        """description of __init__"""
        SocketHandler.__init__(self, host, port)
        self.ca_certs = ca_certs
        self.cert_reqs = cert_reqs
        self.use_tls = use_tls

    def makeSocket(self, timeout=1, **kwargs):
        """makeSocket"""
        sock = SocketHandler.makeSocket(self, timeout=timeout)
        sock.setsockopt(
            socket.SOL_SOCKET, socket.SO_KEEPALIVE, kwargs.get('keep_alive', 1)
        )
        if platform.system() in ('Linux', 'Windows'):
            sock.setsockopt(
                socket.IPPROTO_TCP, socket.TCP_KEEPIDLE,
                kwargs.get('after_idle_sec', 1)
            )
            sock.setsockopt(
                socket.IPPROTO_TCP, socket.TCP_KEEPINTVL,
                kwargs.get('interval_sec', 3)
            )
            sock.setsockopt(
                socket.IPPROTO_TCP, socket.TCP_KEEPCNT,
                kwargs.get('max_fails', 5)
            )

        if self.use_tls is True:
            return ssl.wrap_socket(
                sock, cert_reqs=self.cert_reqs, ca_certs=self.ca_certs
            )
        return sock

    def makePickle(self, record):
        return bytes(self.format(record) + "\n", "UTF-8")


class GELFUDPSocketHandler(DatagramHandler):
    def makePickle(self, record):
        return bytes(self.format(record) + "\n", "UTF-8")



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Use of this source code is governed by a BSD-style
# license that can be found in the LICENSE file.
# Copyright 2019 The logging-gelf Authors. All rights reserved.

"""
Formatters specify the layout of log records in the final output (GELF).
"""

import re
import json
import logging
from logging_gelf.schemas import GelfSchema


class StringJSONEncoder(json.JSONEncoder):
    def encode(self, o):
        """description of encode"""
        res = dict()
        for key, value in o.items():
            if isinstance(value, (list, tuple, dict)):
                res[key] = str(value)
            else:
                res[key] = value
        return json.JSONEncoder.encode(self, res)


class GELFFormatter(logging.Formatter):
    """A GELF formatter to format a :class:`logging.LogRecord` into GELF.

    :param logging_gelf.schemas.GelfSchema schema: The marshmallow schema to
    use to format data.
    :param bool null_character: Append a '\0' at the end of the string. It
    depends on the input used.
    :param list|None exclude_patterns: List of regexp used to exclude keys.
    """

    def __init__(self, schema=GelfSchema, null_character=False,
                 JSONEncoder=json.JSONEncoder, exclude_patterns=None):
        if not issubclass(schema, GelfSchema):
            raise ValueError("Schema MUST inherit from 'GelfSchema'")

        self.schema = schema
        self.null_character = null_character
        self._encoder_cls = JSONEncoder
        exclude_patterns = exclude_patterns if exclude_patterns else list()
        self.exclude_patterns = ['^_{}'.format(x) for x in exclude_patterns]
        logging.Formatter.__init__(self)

    def format(self, record):
        """Format the specified record into json using the schema which MUST
        inherit from :class:`logging_gelf.schemas.GelfSchema`.

        :param logging.LogRecord record: Contains all the information pertinent
        to the event being logged.
        :return: A JSON dump of the record.
        :rtype: str
        """
        data = self.filter_keys(self.schema().dump(record))
        out = json.dumps(data, cls=self._encoder_cls)
        if self.null_character is True:
            out += '\0'
        return out

    def filter_keys(self, data):
        """Filter GELF record keys using exclude_patterns

        :param dict data: Log record has dict
        :return: the filtered log record
        :rtype: dict
        """
        if self.exclude_patterns is None:
            return data

        keys = list(data.keys())
        for pattern in self.exclude_patterns:
            for key in keys:
                if re.match(pattern, key):
                    keys.remove(key)

        return dict(filter(lambda x: x[0] in keys, data.items()))



# /usr/bin/env python
# -*- coding: utf-8 -*-

# Use of this source code is governed by a BSD-style
# license that can be found in the LICENSE file.
# Copyright 2019 The logging-gelf Authors. All rights reserved.

from setuptools import setup, find_packages

setup(
    name='logging-gelf',
    version=open('VERSION', 'r').read().strip(),
    description="Logging bundle to send logs using GELF",
    long_description=open('README.rst', 'r').read().strip(),
    classifiers=["Programming Language :: Python"],
    keywords='',
    author='OVH SAS',
    author_email='github@ovh.net',
    url='https://github.com/ovh/python-logging-gelf',
    license='BSD',
    py_modules=['logging_gelf'],
    include_package_data=True,
    zip_safe=True,
    install_requires=open('requirements.txt', 'r').readlines(),
    packages=find_packages('src'),
    package_dir={'': 'src'},
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
.. codeauthor:: Cédric Dumay <cedric.dumay@gmail.com>


"""
import json
import logging
from logging import config
from logging_gelf.formatters import GELFFormatter
from logging_gelf.schemas import GelfSchema


def _create_formatters(cp):
    """Create and return formatters"""
    flist = cp["formatters"]["keys"]
    if not len(flist):
        return {}
    flist = flist.split(",")
    flist = config._strip_spaces(flist)
    formatters = {}
    for form in flist:
        sectname = "formatter_%s" % form
        fclass = logging.Formatter
        class_name = cp[sectname].get("class")
        if class_name:
            fclass = config._resolve(class_name)
        if issubclass(fclass, GELFFormatter):
            exclude_patterns = cp.get(
                sectname, "exclude_patterns", raw=True, fallback=None
            )
            if exclude_patterns:
                exclude_patterns = eval(exclude_patterns, vars(logging))
            f = fclass(
                config._resolve(cp.get(
                    sectname, "schema", raw=True,
                    fallback="logging_gelf.schemas.GelfSchema"
                )),
                cp.getboolean(
                    sectname, "null_character", raw=True, fallback=False
                ),
                config._resolve(cp.get(
                    sectname, "JSONEncoder", raw=True,
                    fallback="json.JSONEncoder"
                )),
                exclude_patterns
            )
        else:
            fs = cp.get(sectname, "format", raw=True, fallback=None)
            dfs = cp.get(sectname, "datefmt", raw=True, fallback=None)
            stl = cp.get(sectname, "style", raw=True, fallback='%')
            f = fclass(fs, dfs, stl)
        formatters[form] = f
    return formatters


config._create_formatters = _create_formatters
